summaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
authorMartin Pitt <martinpitt@gnome.org>2013-09-27 08:26:58 +0200
committerMartin Pitt <martinpitt@gnome.org>2013-09-27 09:46:02 +0200
commit5c132c683d0ddfc78a58bb4de6582c6a292a067f (patch)
tree67e21437c7473981669f964e0313aac1757bfca7 /src
parent6f9ccd35b3c1d154735e69064fda3ebee63cb2e0 (diff)
downloadupower-5c132c683d0ddfc78a58bb4de6582c6a292a067f.tar.gz
linux: Track power_supply devices by name only instead of full sysfs path
The full native sysfs path of wireless HID battery devices contains a lot of jitter like USB tree layout or sequence numbers which change after every resume from auto-suspend. Plus, there are kernel bugs which don't give proper remove events for those: http://bugzilla.kernel.org/show_bug.cgi?id=62041 As device names within the same subsystem must be unique anyway, and we only use the device name for constructing the D-BUS object path, only consider the device name for the device list native → upower object lookup table, i. e. in up_native_get_native_path(). This fixes the "A handler is already registered for <battery>" assertion crash if a bluetooth device comes back from auto-suspend. This fixes the test_bluetooth_mouse_reconnect() test case, drop the expected failure. https://launchpad.net/bugs/1112907
Diffstat (limited to 'src')
-rwxr-xr-xsrc/linux/integration-test9
-rw-r--r--src/linux/up-native.c13
2 files changed, 14 insertions, 8 deletions
diff --git a/src/linux/integration-test b/src/linux/integration-test
index 82b5ff7..68964ef 100755
--- a/src/linux/integration-test
+++ b/src/linux/integration-test
@@ -211,7 +211,7 @@ class Tests(unittest.TestCase):
self.assertEqual(self.get_dbus_dev_property(ac_up, 'PowerSupply'), True)
self.assertEqual(self.get_dbus_dev_property(ac_up, 'Type'), 1)
self.assertEqual(self.get_dbus_dev_property(ac_up, 'Online'), True)
- self.assertEqual(self.get_dbus_dev_property(ac_up, 'NativePath'), ac)
+ self.assertEqual(self.get_dbus_dev_property(ac_up, 'NativePath'), 'AC')
self.stop_daemon()
# offline AC
@@ -252,7 +252,7 @@ class Tests(unittest.TestCase):
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'EnergyFull'), 60.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'EnergyFullDesign'), 80.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'Voltage'), 12.0)
- self.assertEqual(self.get_dbus_dev_property(bat0_up, 'NativePath'), bat0)
+ self.assertEqual(self.get_dbus_dev_property(bat0_up, 'NativePath'), 'BAT0')
self.stop_daemon()
# offline AC + discharging low battery
@@ -380,7 +380,6 @@ class Tests(unittest.TestCase):
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'EnergyFullDesign'), 132.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'Voltage'), 12.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'Temperature'), 0.0)
- self.assertEqual(self.get_dbus_dev_property(bat0_up, 'NativePath'), bat0)
self.stop_daemon()
def test_battery_energy_charge_mixed(self):
@@ -408,7 +407,6 @@ class Tests(unittest.TestCase):
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'EnergyFull'), 126.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'EnergyFullDesign'), 132.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'Voltage'), 12.0)
- self.assertEqual(self.get_dbus_dev_property(bat0_up, 'NativePath'), bat0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'Percentage'), 40.0)
self.stop_daemon()
@@ -436,7 +434,6 @@ class Tests(unittest.TestCase):
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'EnergyFull'), 126.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'EnergyFullDesign'), 132.0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'Voltage'), 12.0)
- self.assertEqual(self.get_dbus_dev_property(bat0_up, 'NativePath'), bat0)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'PowerSupply'), True)
self.assertEqual(self.get_dbus_dev_property(bat0_up, 'Type'), 2)
@@ -623,8 +620,6 @@ class Tests(unittest.TestCase):
self.assertEqual(self.get_dbus_property('OnLowBattery'), False)
self.stop_daemon()
- # https://launchpad.net/bugs/1112907
- @unittest.expectedFailure
def test_bluetooth_mouse_reconnect(self):
'''bluetooth mouse powerdown/reconnect'''
diff --git a/src/linux/up-native.c b/src/linux/up-native.c
index 5e03309..a700d49 100644
--- a/src/linux/up-native.c
+++ b/src/linux/up-native.c
@@ -31,11 +31,22 @@
* This would be implemented on a Linux system using:
* g_udev_device_get_sysfs_path (G_UDEV_DEVICE (object))
*
* Return value: The native path for the device which is unique, e.g. "/sys/class/power/BAT1"
**/
const gchar *
up_native_get_native_path (GObject *object)
{
+ /* Device names within the same subsystem must be unique. To avoid
+ * treating the same power supply device on variable buses as different
+ * only because e. g. the USB or bluetooth tree layout changed, only
+ * use their name as identification. Also see
+ * http://bugzilla.kernel.org/show_bug.cgi?id=62041 */
+ if (g_strcmp0 (g_udev_device_get_subsystem (G_UDEV_DEVICE (object)), "power_supply") == 0)
+ return g_udev_device_get_name (G_UDEV_DEVICE (object));
+
+ /* we do not expect other devices than power_supply, but provide this
+ * fallback for completeness */
return g_udev_device_get_sysfs_path (G_UDEV_DEVICE (object));
}
@@ -31,11 +31,22 @@
* g_udev_device_get_sysfs_path (G_UDEV_DEVICE (object))
*
- * Return value: The native path for the device which is unique, e.g. "/sys/class/power/BAT1"
+ * Return value: Device name for devices of subsystem "power_supply", otherwise
+ * the native path for the device which is unique.
**/
const gchar *